#include <vector>
#include <algorithm>
#include "bit/CompressedArray.hh"
#include "Timer.hh"

using namespace bit;

int
main(int argc, char *argv[])
{
  CompressedArray a;

  int num_elems = atoi(argv[1]);
  int max_elem = atoi(argv[2]);
  int rounds1 = atoi(argv[3]);
  int rounds2 = atoi(argv[4]);

  fprintf(stderr, "%d elems (max %d)\n", num_elems, max_elem);

  srand(0);
  std::vector<int> vec;
  for (int i = 0; i < num_elems; i++)
    vec.push_back(rand() % max_elem);
  std::sort(vec.begin(), vec.end());
  for (int i = 0; i < (int)vec.size(); i++)
    a.set_grow_widen(i, vec[i]);

  Timer timer;
  int counter;

  timer.reset();
  timer.start();
  counter = 0;
  assert(a.num_elems() == vec.size());
  for (int r = 0; r < rounds1; r++) {
    for (int i = 0; i < a.num_elems(); i++) {
      counter += a.get(i);
      assert(a.get(i) == vec.at(i));
    }
  }
  timer.stop();
  fprintf(stderr, "counter: %d\n", counter);
  fprintf(stderr, "total time: %.2f\n", timer.total_sec());

  a.recursive_optimal_compress();

  timer.reset();
  timer.start();
  assert(a.num_elems() == vec.size());
  counter = 0;
  for (int r = 0; r < rounds2; r++) {
    for (int i = 0; i < a.num_elems(); i++) {
      counter += a.get(i);
      assert(a.get(i) == vec.at(i));
    }
  }
  timer.stop();
  fprintf(stderr, "counter: %d\n", counter);
  fprintf(stderr, "total time: %.2f\n", timer.total_sec());
}
