#ifndef UTIL_HH
#define UTIL_HH

#include <stdexcept>
#include <vector>
#include <stdio.h>

namespace util {

  /** Thrown to report input output errors. */
  class io_error : public std::runtime_error
  {
  public:
    explicit io_error(const std::string&  __arg) 
      : runtime_error(__arg) { }
  };

  template <typename T>
  void write_vec_raw(FILE *file, const std::vector<T> &vec)
  {
    fprintf(file, "%zd:", vec.size());
    size_t ret = fwrite(&vec[0], sizeof(vec[0]), vec.size(), file);
    if (ret != vec.size())
      throw io_error("bit::Kneser::write_vec_raw() failed");
  }

  template <typename T>
  void read_vec_raw(FILE *file, std::vector<T> &vec)
  {
    size_t size;
    {
      int ret = fscanf(file, "%zd:", &size);
      if (ret != 1)
        throw io_error("bit::Kneser::read_vec_raw() failed");
    }
    vec.resize(size);
    if (size == 0)
      return;
    size_t ret = fread(&vec[0], sizeof(vec[0]), vec.size(), file);
    if (ret != vec.size())
      throw io_error("bit::Kneser::read_vec_raw() failed");
  }


  template <typename T>
  void write(FILE *file, const std::vector<T> &vec)
  {
    fprintf(file, "%zd:", vec.size());
    for (size_t i = 0; i < vec.size(); i++)
      util::write(file, vec[i]);
    if (ferror(file))
      throw io_error("bit::Kneser::write() failed");
  }

  template <typename T>
  void read(FILE *file, std::vector<T> &vec)
  {
    size_t size;
    int ret = fscanf(file, "%zd:", &size);
    if (ret != 1)
      throw io_error("bit::Kneser::read() failed");
    vec.resize(size);
    if (size == 0)
      return;
    for (size_t i = 0; i < vec.size(); i++)
      util::read(file, vec[i]);
  }

  template <>
  void write(FILE *file, const std::vector<int> &vec)
  {
    write_vec_raw(file, vec);
  }

  template <>
  void read(FILE *file, std::vector<int> &vec)
  {
    read_vec_raw(file, vec);
  }

  template <>
  void write(FILE *file, const std::vector<float> &vec)
  {
    write_vec_raw(file, vec);
  }

  template <>
  void read(FILE *file, std::vector<float> &vec)
  {
    read_vec_raw(file, vec);
  }

};

#endif /* UTIL_HH */
